"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const Connection_1 = require("./Connection");
const ConnectionNotFoundError_1 = require("./error/ConnectionNotFoundError");
const MysqlDriver_1 = require("../driver/MysqlDriver");
const MissingDriverError_1 = require("./error/MissingDriverError");
const PostgresDriver_1 = require("../driver/PostgresDriver");
const AlreadyHasActiveConnectionError_1 = require("./error/AlreadyHasActiveConnectionError");
/**
 * Connection manager holds all connections made to the databases and providers helper management functions
 * for all exist connections.
 */
class ConnectionManager {
    constructor() {
        // -------------------------------------------------------------------------
        // Properties
        // -------------------------------------------------------------------------
        /**
         * List of connections registered in this connection manager.
         */
        this.connections = [];
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Creates a new connection based on the given connection options and registers a new connection in the manager.
     */
    createAndConnect(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const connection = this.create(options);
            // connect to the database
            yield connection.connect();
            // if option is set - drop schema once connection is done
            if (options.dropSchemaOnConnection)
                yield connection.dropDatabase();
            // if option is set - automatically synchronize a schema
            if (options.autoSchemaCreate)
                yield connection.syncSchema();
            return connection;
        });
    }
    /**
     * Creates a new connection based on the given connection options and registers this connection in the manager.
     */
    create(options) {
        const driver = this.createDriver(options.driver);
        const connection = this.createConnection(options.connectionName || "default", driver);
        if (options.entitySchemaDirectories && options.entitySchemaDirectories.length > 0)
            connection.importEntitySchemaFromDirectories(options.entitySchemaDirectories);
        if (options.entitySchemas)
            connection.importEntitySchemas(options.entitySchemas);
        if (options.entityDirectories && options.entityDirectories.length > 0)
            connection.importEntitiesFromDirectories(options.entityDirectories);
        if (options.entities)
            connection.importEntities(options.entities);
        if (options.subscriberDirectories && options.subscriberDirectories.length > 0)
            connection.importSubscribersFromDirectories(options.subscriberDirectories);
        if (options.subscribers)
            connection.importSubscribers(options.subscribers);
        if (options.namingStrategyDirectories && options.namingStrategyDirectories.length > 0)
            connection.importNamingStrategiesFromDirectories(options.namingStrategyDirectories);
        if (options.namingStrategies)
            connection.importNamingStrategies(options.namingStrategies);
        if (options.usedNamingStrategy && typeof options.usedNamingStrategy === "string")
            connection.useNamingStrategy(options.usedNamingStrategy);
        if (options.usedNamingStrategy && options.usedNamingStrategy instanceof Function)
            connection.useNamingStrategy(options.usedNamingStrategy);
        return connection;
    }
    /**
     * Gets registered connection with the given name.
     * If connection name is not given then it will get a default connection.
     */
    get(name = "default") {
        const connection = this.connections.find(connection => connection.name === name);
        if (!connection)
            throw new ConnectionNotFoundError_1.ConnectionNotFoundError(name);
        return connection;
    }
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    /**
     * Creates a new driver based on the given driver type and options.
     */
    createDriver(options) {
        switch (options.type) {
            case "mysql":
                return new MysqlDriver_1.MysqlDriver(options);
            case "postgres":
                return new PostgresDriver_1.PostgresDriver(options);
            default:
                throw new MissingDriverError_1.MissingDriverError(options.type);
        }
    }
    /**
     * Creates a new connection and registers it in the connection manager.
     */
    createConnection(name, driver) {
        const existConnection = this.connections.find(connection => connection.name === name);
        if (existConnection) {
            if (existConnection.isConnected)
                throw new AlreadyHasActiveConnectionError_1.AlreadyHasActiveConnectionError(name);
            this.connections.splice(this.connections.indexOf(existConnection), 1);
        }
        const connection = new Connection_1.Connection(name, driver);
        this.connections.push(connection);
        return connection;
    }
}
exports.ConnectionManager = ConnectionManager;

//# sourceMappingURL=ConnectionManager.js.map
