"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const ColumnTypes_1 = require("../metadata/types/ColumnTypes");
const moment = require("moment");
/**
 * Provides base functionality for all driver implementations.
 */
class BaseDriver {
    constructor() {
        // -------------------------------------------------------------------------
        // Private Properties
        // -------------------------------------------------------------------------
        this.transactionActive = false;
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Updates rows that match given conditions in the given table.
     */
    update(tableName, valuesMap, conditions) {
        this.checkIfConnectionSet();
        const updateValues = this.escapeObjectMap(valuesMap).join(",");
        const conditionString = this.escapeObjectMap(conditions).join(" AND ");
        const query = `UPDATE ${tableName} SET ${updateValues} ${conditionString ? (" WHERE " + conditionString) : ""}`;
        // console.log("executing update: ", query);
        return this.query(query).then(() => { });
    }
    escapeObjectMap(objectMap) {
        return Object.keys(objectMap).map(key => {
            const value = objectMap[key];
            if (value === null || value === undefined) {
                return key + "=NULL";
            }
            else {
                return key + "=" + this.escape(value);
            }
        });
    }
    /**
     * Insert a new row into given table.
     */
    insert(tableName, keyValues, idColumnName) {
        this.checkIfConnectionSet();
        const columns = Object.keys(keyValues).join(",");
        const values = Object.keys(keyValues).map(key => this.escape(keyValues[key])).join(","); // todo: escape here
        const query = `INSERT INTO ${tableName}(${columns}) VALUES (${values})`;
        return this.query(query).then(result => result.insertId);
    }
    /**
     * Deletes from the given table by a given conditions.
     */
    delete(tableName, conditions) {
        this.checkIfConnectionSet();
        const conditionString = this.escapeObjectMap(conditions).join(" AND ");
        const query = `DELETE FROM ${tableName} WHERE ${conditionString}`;
        return this.query(query).then(() => { });
    }
    /**
     * Starts transaction.
     */
    beginTransaction() {
        return __awaiter(this, void 0, void 0, function* () {
            this.transactionActive = true;
            yield this.query("START TRANSACTION");
        });
    }
    /**
     * Commits transaction.
     */
    commitTransaction() {
        return __awaiter(this, void 0, void 0, function* () {
            this.transactionActive = false;
            yield this.query("COMMIT");
        });
    }
    /**
     * Rollbacks transaction.
     */
    rollbackTransaction() {
        return __awaiter(this, void 0, void 0, function* () {
            this.transactionActive = false;
            yield this.query("ROLLBACK");
        });
    }
    /**
     * Checks if transaction is active.
     */
    isTransactionActive() {
        return this.transactionActive;
    }
    /**
     * Prepares given value to a value to be persisted, based on its column type and metadata.
     */
    preparePersistentValue(value, column) {
        switch (column.type) {
            case ColumnTypes_1.ColumnTypes.BOOLEAN:
                return value === true ? 1 : 0;
            case ColumnTypes_1.ColumnTypes.DATE:
                return moment(value).format("YYYY-MM-DD");
            case ColumnTypes_1.ColumnTypes.TIME:
                return moment(value).format("HH:mm:ss");
            case ColumnTypes_1.ColumnTypes.DATETIME:
                return moment(value).format("YYYY-MM-DD HH:mm:ss");
            case ColumnTypes_1.ColumnTypes.JSON:
                return JSON.stringify(value);
            case ColumnTypes_1.ColumnTypes.SIMPLE_ARRAY:
                return value
                    .map(i => String(i))
                    .join(",");
        }
        return value;
    }
    /**
     * Prepares given value to a value to be persisted, based on its column type and metadata.
     */
    prepareHydratedValue(value, column) {
        switch (column.type) {
            case ColumnTypes_1.ColumnTypes.BOOLEAN:
                return value ? true : false;
            case ColumnTypes_1.ColumnTypes.DATE:
                if (value instanceof Date)
                    return value;
                return moment(value, "YYYY-MM-DD").toDate();
            case ColumnTypes_1.ColumnTypes.TIME:
                return moment(value, "HH:mm:ss").toDate();
            case ColumnTypes_1.ColumnTypes.DATETIME:
                if (value instanceof Date)
                    return value;
                return moment(value, "YYYY-MM-DD HH:mm:ss").toDate();
            case ColumnTypes_1.ColumnTypes.JSON:
                return JSON.parse(value);
            case ColumnTypes_1.ColumnTypes.SIMPLE_ARRAY:
                return value.split(",");
        }
        return value;
    }
    /**
     * Inserts rows into closure table.
     */
    insertIntoClosureTable(tableName, newEntityId, parentId, hasLevel) {
        let sql = "";
        if (hasLevel) {
            sql = `INSERT INTO ${tableName}(ancestor, descendant, level) ` +
                `SELECT ancestor, ${newEntityId}, level + 1 FROM ${tableName} WHERE descendant = ${parentId} ` +
                `UNION ALL SELECT ${newEntityId}, ${newEntityId}, 1`;
        }
        else {
            sql = `INSERT INTO ${tableName}(ancestor, descendant) ` +
                `SELECT ancestor, ${newEntityId} FROM ${tableName} WHERE descendant = ${parentId} ` +
                `UNION ALL SELECT ${newEntityId}, ${newEntityId}`;
        }
        return this.query(sql).then(() => {
            return this.query(`SELECT MAX(level) as level FROM ${tableName} WHERE descendant = ${parentId}`);
        }).then((results) => {
            return results && results[0] && results[0]["level"] ? parseInt(results[0]["level"]) + 1 : 1;
        });
    }
    // -------------------------------------------------------------------------
    // Protected Methods
    // -------------------------------------------------------------------------
    logQuery(query) {
        if (this.connectionOptions.logging && this.connectionOptions.logging.logQueries)
            this.log("executing query: " + query, "log");
    }
    logQueryError(error) {
        if (this.connectionOptions.logging && this.connectionOptions.logging.logFailedQueryError) {
            this.log("error during executing query:", "error");
            this.log(error, "error");
        }
    }
    logFailedQuery(query) {
        if (this.connectionOptions.logging &&
            (this.connectionOptions.logging.logQueries || this.connectionOptions.logging.logOnlyFailedQueries))
            this.log("query failed: " + query, "error");
    }
    log(message, level) {
        if (!this.connectionOptions.logging)
            return;
        if (this.connectionOptions && this.connectionOptions.logging && this.connectionOptions.logging.logger) {
            this.connectionOptions.logging.logger(message, level);
        }
        else {
            switch (level) {
                case "log":
                    console.log(message);
                    break;
                case "debug":
                    console.debug(message);
                    break;
                case "info":
                    console.info(message);
                    break;
                case "error":
                    console.error(message);
                    break;
            }
        }
    }
}
exports.BaseDriver = BaseDriver;

//# sourceMappingURL=BaseDriver.js.map
