"use strict";
const MysqlSchemaBuilder_1 = require("../schema-builder/MysqlSchemaBuilder");
const ConnectionIsNotSetError_1 = require("./error/ConnectionIsNotSetError");
const BaseDriver_1 = require("./BaseDriver");
/**
 * This driver organizes work with mysql database.
 */
class MysqlDriver extends BaseDriver_1.BaseDriver {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    constructor(connectionOptions, mysql) {
        super();
        this.connectionOptions = connectionOptions;
        // -------------------------------------------------------------------------
        // Public Properties
        // -------------------------------------------------------------------------
        this.isResultsLowercase = false;
        // if driver dependency is not given explicitly, then try to load it via "require"
        if (!mysql && require) {
            try {
                mysql = require("mysql");
            }
            catch (e) {
                throw new Error("Mysql package has not been found installed. Try to install it: npm install mysql --save");
            }
        }
        else {
            throw new Error("Cannot load mysql driver dependencies. Try to install all required dependencies.");
        }
        this.mysql = mysql;
    }
    // -------------------------------------------------------------------------
    // Getter Methods
    // -------------------------------------------------------------------------
    /**
     * Access to the native implementation of the database.
     */
    get native() {
        return this.mysql;
    }
    /**
     * Access to the native connection to the database.
     */
    get nativeConnection() {
        return this.mysqlConnection;
    }
    /**
     * Database name to which this connection is made.
     */
    get db() {
        if (this.mysqlConnection && this.mysqlConnection.config.database)
            return this.mysqlConnection.config.database;
        if (this.connectionOptions.database)
            return this.connectionOptions.database;
        throw new Error("Cannot get the database name. Since database name is not explicitly given in configuration " +
            "(maybe connection url is used?), database name cannot be retrieved until connection is made.");
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Creates a schema builder which can be used to build database/table schemas.
     */
    createSchemaBuilder() {
        return new MysqlSchemaBuilder_1.MysqlSchemaBuilder(this);
    }
    /**
     * Performs connection to the database based on given connection options.
     */
    connect() {
        this.mysqlConnection = this.mysql.createConnection({
            host: this.connectionOptions.host,
            user: this.connectionOptions.username,
            password: this.connectionOptions.password,
            database: this.connectionOptions.database,
            port: this.connectionOptions.port
        });
        return new Promise((ok, fail) => {
            this.mysqlConnection.connect((err) => err ? fail(err) : ok());
        });
    }
    /**
     * Closes connection with database.
     */
    disconnect() {
        this.checkIfConnectionSet();
        return new Promise((ok, fail) => {
            this.mysqlConnection.end((err) => err ? fail(err) : ok());
        });
    }
    /**
     * Escapes given value.
     */
    escape(value) {
        return this.mysqlConnection.escape(value);
    }
    /**
     * Executes a given SQL query.
     */
    query(query, parameters) {
        this.checkIfConnectionSet();
        this.logQuery(query);
        return new Promise((ok, fail) => this.mysqlConnection.query(query, parameters, (err, result) => {
            if (err) {
                this.logFailedQuery(query);
                this.logQueryError(err);
                fail(err);
            }
            else {
                ok(result);
            }
        }));
    }
    /**
     * Clears all tables in the currently connected database.
     */
    clearDatabase() {
        this.checkIfConnectionSet();
        const disableForeignKeysCheckQuery = `SET FOREIGN_KEY_CHECKS = 0;`;
        const dropTablesQuery = `SELECT concat('DROP TABLE IF EXISTS ', table_name, ';') AS q FROM ` +
            `information_schema.tables WHERE table_schema = '${this.db}';`;
        const enableForeignKeysCheckQuery = `SET FOREIGN_KEY_CHECKS = 1;`;
        return this.query(disableForeignKeysCheckQuery)
            .then(() => this.query(dropTablesQuery))
            .then(results => Promise.all(results.map(q => this.query(q["q"]))))
            .then(() => this.query(enableForeignKeysCheckQuery))
            .then(() => { });
    }
    buildParameters(sql, parameters) {
        const builtParameters = [];
        Object.keys(parameters).forEach((key, index) => {
            // const value = this.parameters[key] !== null && this.parameters[key] !== undefined ? this.driver.escape(this.parameters[key]) : "NULL";
            sql = sql.replace(new RegExp(":" + key, "g"), (str) => {
                builtParameters.push(parameters[key]);
                return "?";
            }); // todo: make replace only in value statements, otherwise problems
        });
        return builtParameters;
    }
    replaceParameters(sql, parameters) {
        Object.keys(parameters).forEach((key, index) => {
            // const value = parameters[key] !== null && parameters[key] !== undefined ? this.driver.escape(parameters[key]) : "NULL";
            sql = sql.replace(new RegExp(":" + key, "g"), (str) => {
                return "?";
            }); // todo: make replace only in value statements, otherwise problems
        });
        return sql;
    }
    // -------------------------------------------------------------------------
    // Protected Methods
    // -------------------------------------------------------------------------
    checkIfConnectionSet() {
        if (!this.mysqlConnection)
            throw new ConnectionIsNotSetError_1.ConnectionIsNotSetError("mysql");
    }
}
exports.MysqlDriver = MysqlDriver;

//# sourceMappingURL=MysqlDriver.js.map
