/*!
 
 */
"use strict";
const ConnectionManager_1 = require("./connection/ConnectionManager");
const MetadataArgsStorage_1 = require("./metadata-args/MetadataArgsStorage");
// -------------------------------------------------------------------------
// Global Container
// -------------------------------------------------------------------------
/**
 * Container to be used by this library for inversion control. If container was not implicitly set then by default
 * container simply creates a new instance of the given class.
 */
let container = new (class {
    constructor() {
        this.instances = [];
    }
    get(someClass) {
        if (!this.instances[someClass])
            this.instances[someClass] = new someClass();
        return this.instances[someClass];
    }
}
)();
/**
 * Sets container to be used by this library.
 *
 * @param iocContainer
 */
function useContainer(iocContainer) {
    container = iocContainer;
}
exports.useContainer = useContainer;
/**
 * Gets the IOC container used by this library.
 */
function getFromContainer(someClass) {
    return container.get(someClass);
}
exports.getFromContainer = getFromContainer;
// -------------------------------------------------------------------------
// Global Metadata Storage
// -------------------------------------------------------------------------
/**
 * Default metadata storage used as singleton and can be used to storage all metadatas in the system.
 */
let metadataArgsStorage;
/**
 * Gets metadata args storage.
 */
function getMetadataArgsStorage() {
    if (!metadataArgsStorage && container) {
        metadataArgsStorage = container.get(MetadataArgsStorage_1.MetadataArgsStorage);
    }
    else if (!metadataArgsStorage) {
        metadataArgsStorage = new MetadataArgsStorage_1.MetadataArgsStorage();
    }
    return metadataArgsStorage;
}
exports.getMetadataArgsStorage = getMetadataArgsStorage;
// -------------------------------------------------------------------------
// Global Connection Manager
// -------------------------------------------------------------------------
/**
 * Default export. Global connection manager.
 */
let connectionManager;
/**
 * Gets a ConnectionManager which creates connections.
 */
function getConnectionManager() {
    if (!connectionManager && container) {
        connectionManager = container.get(ConnectionManager_1.ConnectionManager);
    }
    else if (!connectionManager) {
        connectionManager = new ConnectionManager_1.ConnectionManager();
    }
    return connectionManager;
}
exports.getConnectionManager = getConnectionManager;
/**
 * Allows to quickly create a connection based on the given options. Uses ConnectionManager.
 */
function createConnection(options) {
    return getConnectionManager().createAndConnect(options);
}
exports.createConnection = createConnection;
// -------------------------------------------------------------------------
// Commonly Used exports
// -------------------------------------------------------------------------
var Connection_1 = require("./connection/Connection");
exports.Connection = Connection_1.Connection;
var ConnectionManager_2 = require("./connection/ConnectionManager");
exports.ConnectionManager = ConnectionManager_2.ConnectionManager;
var MysqlDriver_1 = require("./driver/MysqlDriver");
exports.MysqlDriver = MysqlDriver_1.MysqlDriver;
var QueryBuilder_1 = require("./query-builder/QueryBuilder");
exports.QueryBuilder = QueryBuilder_1.QueryBuilder;
var EntityManager_1 = require("./entity-manager/EntityManager");
exports.EntityManager = EntityManager_1.EntityManager;
var Repository_1 = require("./repository/Repository");
exports.Repository = Repository_1.Repository;

//# sourceMappingURL=index.js.map
